#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------


. ../Common/lib.ksh

#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi
MODULE="CDR"
init_global_variables

#############################

function get_kpi {

clear

echo ""; echo ""; echo ""
echo "    ###############################################"
echo "    #                                             #"
echo "    #            - REVENUE ASSURANCE -            #"
echo "    #                xDR Assurance                #"
echo "    #                                             #"
echo "    #              Calculate KPI Value)           #"
echo "    #                                             #"
echo "    ###############################################"

echo ""
echo ""
echo "Enter the KPI Id to be calculated:"
echo "----------------------------------"
correct_kpi=0
while [ $correct_kpi -ne 1 ]
do
   echo "KPI Id:"
   read kpi
   correct_kpi=1
done

}

#############################

function rollback_kpi {

  if [ $1 -eq 0 ]
  then
	get_kpi;
  fi
  if [[ $running_mode = "interactive" ]];then
     clear
  fi
 
  # Prepare name of the log file
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/rollback_KPI_${date}.log
  touch $logfile
  abort=0

  echo "##############################"  
  echo "##############################"  >> $logfile
  echo "KPI Rollback" 
  echo "KPI Rollback" >> $logfile
  echo "##############################" 
  echo "##############################" >> $logfile
  echo "  Starting the KPI Rollback..." 
  echo "  Starting the KPI Rollback..." >> $logfile
  echo "  -----------------------------------------" 
  echo "  -----------------------------------------" >> $logfile

  validate_RA_DB_available
  if [ $abort = 0 ]; then
  	# No Error - Continue
	echo "  Executing P_KPI_ROLLBACK($kpi)"
	echo "  Executing P_KPI_ROLLBACK($kpi)" >> $logfile
	result=`sqlplus -s /NOLOG << ENDOFSQL
	connect ${UID_RA}/${PASS_RA}@${DB_RA}
	execute P_KPI_ROLLBACK($kpi);
	COMMIT;
	exit;
ENDOFSQL
`
	#look for Oracle error code
	search_and_log_ORACLE_error ${DB_RA}
        if [ $abort = 0 ]; then
	  	# No Error
		echo "    => Success"
		echo "    => Success" >> $logfile
        fi
  fi
  finish_log_and_end

}

#############################

function calculate_kpi {

  if [ $1 -eq 0 ]
  then
	get_kpi;
  fi
  if [[ $running_mode = "interactive" ]];then
     clear
  fi
 
  # Prepare name of the log file
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/calculating_KPI_${date}.log
  touch $logfile
  abort=0

  echo "##############################"  
  echo "##############################"  >> $logfile
  echo "KPI Calculation" 
  echo "KPI Calculation" >> $logfile
  echo "##############################" 
  echo "##############################" >> $logfile
  echo "  Starting the KPI Calculation..." 
  echo "  Starting the KPI Calculation..." >> $logfile
  echo "  -----------------------------------------" 
  echo "  -----------------------------------------" >> $logfile

  validate_RA_DB_available
  if [ $abort = 0 ]; then

  	# No Error - Continue
	if [ $1 -ge 0 ]
	then
		echo "  Executing P_KPI_EXECUTE($kpi)"
		echo "  Executing P_KPI_EXECUTE($kpi)" >> $logfile
		result=`sqlplus -s /NOLOG << ENDOFSQL
		connect ${UID_RA}/${PASS_RA}@${DB_RA}
		execute P_KPI_EXECUTE($kpi);
		COMMIT;
		exit;
ENDOFSQL
`
	else
		echo "  Executing P_KPI_EXECUTE_ALL"
		echo "  Executing P_KPI_EXECUTE_ALL" >> $logfile
		result=`sqlplus -s /NOLOG << ENDOFSQL
		connect ${UID_RA}/${PASS_RA}@${DB_RA}
		execute P_KPI_EXECUTE_ALL;
		COMMIT;
		exit;
ENDOFSQL
`
	fi

	#look for Oracle error codes 
 	search_and_log_ORACLE_error ${DB_RA}
	if [ $abort = 0 ]; then
	  	# No Error
		echo "    => Success"
		echo "    => Success" >> $logfile
        fi

  fi
  finish_log_and_end
}

#############################

choice=-1

if [[ $running_mode = "batch" ]];then
    kpi=$1
    calculate_kpi $kpi
else

    while [ $choice -ne 0 ]
    do

        clear

        echo ""; echo ""; echo ""
        echo "    ###############################################"
        echo "    #                                             #"
        echo "    #            - REVENUE ASSURANCE -            #"
        echo "    #                xDR Assurance                #"
        echo "    #                                             #"
        echo "    #              Calculate KPI Value            #"
        echo "    #                                             #"
        echo "    ###############################################"
        echo ""
        echo ""
        echo "Available options:"
        echo "------------------"
        echo "  1) Calculate a particular KPI"
        echo "  2) Calculate ALL KPIs"
        echo ""
        echo "  9) Rollback the execution of a particular KPI"
        echo ""
        echo "  0) Cancel"
        echo ""
        echo "Select your option: "
        read choice

        case "$choice" in
         1 ) calculate_kpi 0 ;;
         2 ) calculate_kpi -1 ;;
         9 ) rollback_kpi 0 ;;
         0 ) clear; exit 1 ;;
         *)  clear; echo "Wrong option entered"; echo "Press Enter"; read unused; choice=-1;;
        esac

    done

fi

