#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------


. ../Common/lib.ksh

#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi

MODULE="CDR"
init_global_variables
SOURCE="            (Loading Script Menu)            ";

# Post Processing Procedure
oracle_proc_name_normal=P_CDR_MAIN_LOAD
oracle_proc_name_rollback=P_CDR_MAIN_ROLLBACK

read_resource_file

#############################

function get_loading_mode {

cancontinue=0;
abandon=0;
mode=-1;

while [ $cancontinue -ne 1 ]
do

clear

echo ""; echo ""; echo ""
echo "    ###############################################"
echo "    #                                             #"
echo "    #            - REVENUE ASSURANCE -            #"
echo "    #                xDR Assurance                #"
echo "    #                                             #"
echo "    #              Load raw CDR Files             #"
echo "    #                                             #"
echo "    ###############################################"
echo ""
echo ""
echo "Loading mode:"
echo "-------------"
echo "  1) Normal (update graphs)"
echo "  2) Load only (graphs not updated)"
echo "  9) Rollback" 
echo ""
echo "  0) Cancel"
echo ""
echo "Select your option: "
read mode

if [[ $mode = 0 ]]; then
        cancontinue=1;
	abandon=1;
else
  if [[ $mode == +([0-9]) ]]; then
   if [[ ($mode -lt 3)  || ($mode = 9) ]]; then
	cancontinue=1
   else 
        clear; 
        echo "Wrong option entered"; 
        echo "Press Enter"; 
        read unused; 
        mode=-1;
   fi
  else
   clear; 
   echo "Wrong option entered"; 
   echo "Press Enter"; 
   read unused; 
   mode=-1;
 fi
fi
done

}

#############################

function load {
 
  if [[ $running_mode = "interactive" ]];then
     clear
  fi
  abort=0

  start_and_prepare_load_logfile $1
  print_header $1

  echo "  Starting the Parsing/Decoding..." 
  echo "  Starting the Parsing/Decoding..." >> $logfile
  echo "  --------------------------------" 
  echo "  --------------------------------" >> $logfile

  # Step 1: Parsing the file whenever necessary.. otherwise just a copy

  nb_file=0
  for myfile in $(ls ${ready_dir}/${main_directory[$1]}); do
     if [[ -d  ${ready_dir}/${main_directory[$1]}/$myfile ]];then
	# This is a directory.. skip
        unused=1
     else
	((nb_file=nb_file+1))
	echo "  [$myfile]"
	echo "  [$myfile]" >> $logfile
	result=`${parsing_prg_dir}/${parsing_prg[$1]} ${ready_dir}/${main_directory[$1]}/$myfile ${parsed_dir}/${main_directory[$1]}/${myfile}.parsed`
	echo "$result"
	echo "$result" >> $logfile

  # Step 2: Moving the original file to 'processed' directory for archiving
	mv ${ready_dir}/${main_directory[$1]}/$myfile ${ready_dir}/${main_directory[$1]}/processed
	echo "    => Raw File moved to 'processed' directory"
	echo "    => Raw File moved to 'processed' directory" >> $logfile
     fi
  done
  if [[ $nb_file = 0 ]];then
	echo "    <<No File to Process>>"
	echo "    <<No File to Process>>" >> $logfile
  else
	echo "  --------------------------------"
	echo "  --------------------------------" >> $logfile
	echo "  Nb Files Processed: ${nb_file}"
	echo "  Nb Files Processed: ${nb_file}" >> $logfile
  fi
  echo "  --------------------------------"
  echo "  --------------------------------" >> $logfile
  echo "  Done."
  echo "  Done." >> $logfile 
  echo ""
  echo "" >> $logfile  

  # Step 3: Loading the parsed file in database
  echo "  Starting the Uploading in database..."
  echo "  Starting the Uploading in database..." >> $logfile 
  echo "  -------------------------------------"
  echo "  -------------------------------------" >> $logfile 

  # Verify availability of database before attempting loading anything
  validate_RA_DB_available

  if [ $abort = 0 ]; then
       # No Error - Continue
       # Truncate the temporary tables
  	result=`sqlplus -s /NOLOG << ENDOFSQL
  	connect ${UID_RA}/${PASS_RA}@${DB_RA}
  	truncate table SQLLDR_CDR_${oracle_table[$1]};
  	truncate table LOAD_CDR_${oracle_table[$1]};
  	exit;
ENDOFSQL
`
  	nb_file=0
  	for myfile in $(ls ${parsed_dir}/${main_directory[$1]}); do
  	   abort=0
           if [[ -d  ${parsed_dir}/${main_directory[$1]}/$myfile ]];then
	      # This is a directory.. skip
              unused=1
           else
		((nb_file=nb_file+1))
		echo "  [${myfile}]"
		echo "  [${myfile}]" >> $logfile 
        	command_line="sqlldr "
        	command_line="$command_line control=${sqlldr_home_dir}/ctl/${main_directory[$1]}.ctl "
        	command_line="$command_line data=${parsed_dir}/${main_directory[$1]}/${myfile} "
        	command_line="$command_line log=${sqlldr_home_dir}/log/${main_directory[$1]}/${myfile}.log "
        	command_line="$command_line bad=${sqlldr_home_dir}/bad/${main_directory[$1]}/${myfile}.bad  "
		result=`$command_line > $sqlldrlogfile 2>&1 << ENDOFSQL
		${UID_RA}/${PASS_RA}@${DB_RA}
ENDOFSQL
`

  	       # Step 4:  Finding the number of successfully loaded records
		command_line="grep -h successfully "
        	command_line="$command_line ${sqlldr_dir}/log/${main_directory[$1]}/${myfile}.log"
		result=`$command_line` 

		if [ "$result" ]
		then
			echo "    => ${result}" | sed 's/>   /> /' 
			echo "    => ${result}" | sed 's/>   /> /' >> $logfile

	  		# Step 5: Moving the loaded file to 'loaded' directory
			mv ${parsed_dir}/${main_directory[$1]}/$myfile ${loaded_dir}/${main_directory[$1]}/$myfile.loaded
			echo "    => File moved to 'loaded' directory"
			echo "    => File moved to 'loaded' directory" >> $logfile
		else
	  		# Couldn't find the result of the number of rows loaded=problem. Not moving the file
			echo "    => ERROR loading in database. Check the sqlldr log."
			echo "    => ERROR loading in database. Check the sqlldr log." >> $logfile
		fi
     	   fi
        done
  	if [[ $nb_file = 0 ]];then
		echo "    <<No File to Process>>"
		echo "    <<No File to Process>>" >> $logfile
  	else
		echo "  -------------------------------------"
		echo "  -------------------------------------" >> $logfile
		echo "  Nb Files Processed: ${nb_file}"
		echo "  Nb Files Processed: ${nb_file}" >> $logfile
		echo "  -------------------------------------"
  		echo "  -------------------------------------" >> $logfile
  		echo "  Done."
  		echo "  Done." >> $logfile

  		echo ""
  		echo "" >> $logfile

      		if [[ $2 -ne 2 ]]; then

      			# Step 6: Execute the SQL-Procedures to load in the final tables

			if [[ $2 = 1 ]]; then
				oracle_proc_name=${oracle_proc_name_normal}
			fi
			if [[ $2 = 9 ]]; then
				oracle_proc_name=${oracle_proc_name_rollback}
			fi

  			echo "  Starting the Post-Processing..." 
			echo "  Starting the Post-Processing..." >> $logfile
  			echo "  ---------------------------------"
  			echo "  ---------------------------------" >> $logfile

			# Execute the post-processing
			echo "  Executing ${oracle_proc_name}('${oracle_table[$1]}')"
			echo "  Executing ${oracle_proc_name}('${oracle_table[$1]}')" >> $logfile
			result=`sqlplus -s /NOLOG << ENDOFSQL
        		connect ${UID_RA}/${PASS_RA}@${DB_RA}
			execute ${oracle_proc_name}('${oracle_table[$1]}');
			exit;
ENDOFSQL
`
			#look for Oracle error codes  
  			search_and_log_ORACLE_error ${DB_RA}

                        if [ $abort = 0 ]; then	

	       			# No Error
				echo "    => Success"
				echo "    => Success" >> $logfile
	 		fi 
      		fi #-- End test on loading mode
	
  	fi #-- End test on nb-files to load

  fi #-- End test on RA-DB availability
  finish_log_and_end
}

#############################

choice=-1

if [[ $running_mode = "batch" ]];then
  load $1 1
else

  while [ $choice -ne 0 ]
  do
    display_sources
    read choice

    if [[ $choice = 0 ]]; then
	clear;
	exit 1;
    fi
    if [[ $choice == +([0-9]) ]]; then
        if [[ $choice -le $nbsources ]]; then
	   get_loading_mode;
	   if [[ $abandon = 0 ]]; then
		load $choice $mode;
	   else
   		clear; 
		echo "Loading Cancelled"; 
		echo "Press Enter"; 
		read unused; 
		choice=-1;
	   fi
        else	
  	   clear; 
	   echo "Wrong option entered"; 
	   echo "Press Enter"; 
	   read unused; 
	   choice=-1;
        fi
    else
        clear; 
        echo "Wrong option entered"; 
        echo "Press Enter"; 
        read unused; 
       choice=-1;
    fi
  done
fi
