#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

. ../Common/lib.ksh

#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi

MODULE="CDR"

init_global_variables
read_resource_file

#############################

function purge_files_older_than_3_days {

  for myfile in $(find ${raw_dir}/${main_directory[$1]}/processed -mtime +3); do
     if [[ -d  $myfile ]];then
        # This is a directory.. skip
        unused=1
     else
        rm -f $myfile
     fi
  done
  for myfile in $(find ${loaded_dir}/${main_directory[$1]} -mtime +3); do
     if [[ -d  $myfile ]];then
        # This is a directory.. skip
        unused=1
     else
        rm -f $myfile
     fi
  done

}

#############################

function purge {
 
  # Prepare name of the log file
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/purge_CDR_files_${date}.log
  touch $logfile

  clear
  echo "##############################"  
  echo "##############################"  >> $logfile
  echo "Purge Files"
  echo "Purge Files" >> $logfile
  echo "##############################" 
  echo "##############################" >> $logfile
  echo "  Starting the Purge..." 
  echo "  Starting the Purge..." >> $logfile
  echo "  ---------------------" 
  echo "  ---------------------" >> $logfile

  sourceid=1;
  while [[ $sourceid -le $nbsources ]]; do
     purge_files_older_than_3_days $sourceid
     ((sourceid=sourceid+1));
  done

  purge_logfiles

  echo "  ---------------------"
  echo "  ---------------------" >> $logfile
  echo "  Done."
  echo "  Done." >> $logfile

  echo ""
  echo "" >> $logfile
  if [[ $running_mode = "interactive" ]];then
     echo "Press Enter to return to the main menu"
     read unused
  fi
}

#############################

choice=-1

if [[ $running_mode = "batch" ]];then
    purge
else

    while [ $choice -ne 0 ]
    do

         clear

         echo ""; echo ""; echo ""
         echo "    ###############################################"
         echo "    #                                             #"
         echo "    #            - REVENUE ASSURANCE -            #"
         echo "    #                xDR Assurance                #"
         echo "    #                                             #"
         echo "    #                 Purge Files                 #"
         echo "    #                                             #"
         echo "    ###############################################"
         echo ""
         echo ""
         echo "Confirmation :"
         echo "-------------"
         echo "This will erase the files permanently"
         echo ""
         echo "  1) Confirm"
         echo "  0) Cancel"
         echo ""
         echo "Select your option: "
         read choice

         case "$choice" in
         1 ) purge ; choice=0;;
         0 ) clear; exit 1 ;;
         *)  clear; echo "Wrong option entered"; echo "Press Enter"; read unused; choice=-1;;
         esac

    done
fi
