#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------


function extract_remote_host {

  #----------------------------------------------------------------------
  #-- Extract proc to be used when the CDR files have to be collected 
  #-- on a different server (by FTP)
  #-- (files are not deleted or moved once collected)
  #----------------------------------------------------------------------
  #--
  #-- Parameters required in the resource file:
  #--      remote-host: Remote server where files have to be collected
  #--      remote-user: Login to be used to connect to the remote server
  #--      remote-password: Password to be used to connect to the remote server
  #--      remote-port: TCP/IP port to be used to connect (Optional)
  #--      remote-directory: Directory where files are located on the remote server
  #--      file-pattern: Pattern that will be used to identify the files to be collected
  #--      rsh-allowed: Can you execute a remote-shell command on the remote server (Optional)
  #--                   1 (Yes), 0 (No. Default)
  #--      remote-delete: Set to 1 if you want to delete the files on the remote server once downloaded (Optional)
  #--                   1 (Yes), 0 (No. Default)
  #--
  #----------------------------------------------------------------------

  # Get the last file collected
  last_file=`tail -1 ${extracted_dir}/${main_directory[$1]}/processed/${my_extracted_files}`

  if [ $MODULE = "CDR" ]; then
	filepattern="${file_pattern[$1]}"
  else
	filepattern=$2
  fi
  if [ "${rsh_allowed[$1]}" = "1" ]; then

	  # Generate the list of files in the remote host
	 command="rsh -l "
	 command="$command ${remote_user[$1]}"
	 command="$command ${remote_host[$1]}"
	 command="$command 'cd ${remote_directory[$1]};"
	 command="$command find . -name \"${filepattern}\" \( -newer ./${last_file} \)'"

	 result=`eval "${command}"`

   else
         command="(sleep 1;"
         command="$command echo 'user ${remote_user[$1]} ${remote_password[$1]}';"
         command="$command sleep 1;"
         command="$command echo 'cd ${remote_directory[$1]}';"
         command="$command sleep 1;"
         command="$command echo 'dir ${filepattern}';"
         command="$command sleep 1;"
         command="$command echo 'bye';"
         command="$command )"

         result=`eval "${command}" | ftp -n ${remote_host[$1]} ${remote_port[$1]} | awk '{print $9}' | grep -E ${filepattern} `
   fi

   nb_file=0
   for myfile in  $result ; do

	file_name=`echo $myfile | sed 's/.\///'`
        copy=1
        if [ $MODULE = "CDR" ]; then
              	# See if it has been processed already or not...
               	result=`grep ${file_name} ${extracted_dir}/${main_directory[$1]}/processed/${my_extracted_files}`
               	if [ "$result" ]
               	then
                       	# File Already extracted... Do not download again 
			copy=0
               	fi
	fi
	if [ $copy = 1 ]; then
                # New File - Continue
                ((nb_file=nb_file+1))
                 echo "  [${file_name}]"
                 echo "  [${file_name}]" >> $logfile

                 # Copying the file to the 'extracted' directory
		 command="(sleep 2;"
         	 command="$command echo 'user ${remote_user[$1]} ${remote_password[$1]}';"
         	 command="$command sleep 1;"
	 	 command="$command echo 'cd ${remote_directory[$1]}';"
	 	 command="$command sleep 1;"
		 command="$command echo 'lcd ${extracted_dir}/${main_directory[$1]}';"
	 	 command="$command sleep 1;"
		 command="$command echo 'bin';"
	 	 command="$command sleep 1;"
		 command="$command echo 'get ${file_name}';"
	 	 command="$command sleep 1;"
	 	 command="$command echo 'bye';"
	 	 command="$command )"

		 ftp_result=`eval "${command}" | ftp -n ${remote_host[$1]} ${remote_port[$1]}`

	  	 # Moving the parsed file to 'raw' directory
		 mv ${extracted_dir}/${main_directory[$1]}/${file_name} ${raw_dir}/${main_directory[$1]}
		 echo "    => File moved to 'raw' directory"
		 echo "    => File moved to 'raw' directory" >> $logfile

		 # Registering the file collected
	         if [ $MODULE = "CDR" ]; then
		 	echo "${file_name}" >> ${extracted_dir}/${main_directory[$1]}/processed/${my_extracted_files}
		 fi
		 if [ "${remote_delete[$1]}" = "1" ]; then

                	command="(sleep 1;"
                 	command="$command echo 'user ${remote_user[$1]} ${remote_password[$1]}';"
                 	command="$command sleep 1;"
                	command="$command echo 'cd ${remote_directory[$1]}';"
                	command="$command sleep 1;"
                	command="$command echo 'delete ${file_name}';"
                	command="$command sleep 1;"
                	command="$command echo 'bye';"
                	command="$command )"

                	ftp_result=`eval "${command}" | ftp -n ${remote_host[$1]} ${remote_port[$1]} `

		 	echo "    => File deleted in remote server"
		 	echo "    => File deleted in remote server" >> $logfile
		 fi
        fi
   done

}
