#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

function init_global_variables {

  if [ -z "$UID_RA" ] ; then
	clear
	echo "------ ERROR ------";
	echo "You need to set the following environment variable first:";
	echo "      UID_RA";
	echo "(username to be used to connect to the RA database)";
	exit 1;
  fi
  if [ -z "$DB_RA" ] ; then
        clear
        echo "------ ERROR ------";
        echo "You need to set the following environment variable first:";
        echo "      DB_RA";
        echo "(SID of the RA database)";
        exit 1;
  fi
 
  while read line ; do
        PASS_RA=$line;
  done < .PASS_RA

  while read line ; do
        PASS_BILLING=$line;
  done < .PASS_BILLING

  # Directories
  root_dir=../../UnixStructure
  main_dir=${root_dir}/${MODULE}
  logfile_dir=${main_dir}/log
  extracted_dir=${main_dir}/extracted
  raw_dir=${main_dir}/raw
  if [ $MODULE = "CDR" ]; then
	ready_dir=${raw_dir}
  else
  	ready_dir=${main_dir}/ready
  fi
  loaded_dir=${main_dir}/loaded
  extract_query_dir=${main_dir}/bin/extract
  parsing_prg_dir=${main_dir}/bin
  sqlldr_dir=${main_dir}/bin/sqlldr
  parsed_dir=${main_dir}/parsed
  resource_dir=${main_dir}/bin/loading
  sqlldr_home_dir=${main_dir}/bin/sqlldr
  loads_dir=${main_dir}/loads

  my_extracted_files=list_extracted_files.dat

  source_name[0]="";
  main_directory[0]="";
  source_directory[0]="";
  remote_host[0]="";
  remote_user[0]="";
  remote_password[0]="";
  remote_port[0]="";
  remote_delete[0]="";
  file_pattern[0]="";
  remote_directory[0]="";
  parsing_prg[0]="";
  oracle_table[0]="";
  rsh_allowed[0]="0";
  SQLS_UID[0]="";
  SQLS_PASS[0]="";
  SQLS_URL[0]="";
  ReconciliationId[0]="";
  Reconciliation[0]="";
  Platform_A[0]="";
  Platform_B[0]="";
  no_duplicate_check[0]="No";

  case "$MODULE" in
	"CDR")  
		id_table="REF_CDR_CONTROL_POINTS"
		id_field="CONTROL_POINT_ID" 
		module_description="                xDR Assurance                "
		;;
	"BIV")  
		id_table="REF_BIV_SOURCES" 
		id_field="SOURCE_ID"
		module_description="              Billing Validation             "
		;;
	"SDR")  
		id_table="REF_SDR_PLATFORMS" 
		id_field="PLATFORM_ID"
		module_description="            Subscriber/Data Integrity        "
		;;
  esac

}

#-----------------------------------------------------------------------------------------

function start_and_prepare_extract_logfile {

  if [[ $running_mode = "interactive" ]];then
     clear
  fi

  abort=0
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/extracting_${main_directory[$1]}_${date}.log
  touch $logfile

}

#-----------------------------------------------------------------------------------------

function start_and_prepare_load_logfile {

  if [[ $running_mode = "interactive" ]];then
     clear
  fi

  abort=0
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/loading_${main_directory[$1]}_${date}.log
  sqlldrlogfile=${logfile_dir}/sqlldr_${main_directory[$1]}_${date}.log
  touch $logfile
  touch $sqlldrlogfile

}

#-----------------------------------------------------------------------------------------

function print_header {

  echo "##############################"
  echo "##############################"  >> $logfile
  echo "[$1] ${source_name[$1]} "
  echo "[$1] ${source_name[$1]} " >> $logfile
  echo "##############################"
  echo "##############################" >> $logfile

}

#-----------------------------------------------------------------------------------------

function validate_RA_DB_available {

  validate_ORACLE_DB_available ${UID_RA} ${PASS_RA} ${DB_RA}

}

#-----------------------------------------------------------------------------------------

function validate_ORACLE_DB_available {
 
  result=`sqlplus -s /NOLOG << ENDOFSQL
  connect $1/$2@$3
 	set heading off feedback off verify off
  	select sysdate from dual;
	exit;
ENDOFSQL
`
  search_and_log_ORACLE_error $3

}

#-----------------------------------------------------------------------------------------

function search_and_log_ORACLE_error {

  ora_error_found=`echo "$result" | grep 'ORA' | sed 's/.*ORA-/    => ORA-/' `
  if [ "$ora_error_found" ]
  then
 	# Error found... Log the Error and skip the rest of the processing (abort=1)
	echo "    => ERROR connecting to the $1 database/executing the query/procedure. Process aborted."
	echo "    => ERROR connecting to the $1 database/executing the query/procedure. Process aborted." >> $logfile 
	echo "$ora_error_found"
	echo "$ora_error_found" >> $logfile
	abort=1
  fi

  sp_error_found=`echo "$result" | grep 'SP' | sed 's/.*SP/    => SP/'`
  if [ "$sp_error_found" ]
  then
        # Error found... Log the Error and skip the rest of the processing (abort=1)
        if [ $abort = 0 ]; then
             echo "    => ERROR connecting to the $1 database/executing the query/procedure. Process aborted."
             echo "    => ERROR connecting to the $1 database/executing the query/procedure. Process aborted." >> $logfile
	fi
        echo "$sp_error_found"
        echo "$sp_error_found" >> $logfile
        abort=1
  fi
  if [ $abort = 1 ]; then
        # Log full Oracle-Error in logfile
        echo "    [--- Full Oracle Error ---]" >> $logfile
        echo "${result}" >> $logfile
        echo "    [-------------------------]" >> $logfile
  fi

}

#-----------------------------------------------------------------------------------------

function get_last_extract_dt {

  echo "  Retrieving the last extract date..."
  echo "  Retrieving the last extract date..."  >> $logfile
  echo "  --------------------------------"
  echo "  --------------------------------" >> $logfile

  result=`sqlplus -s /NOLOG << ENDOFSQL
        connect ${UID_RA}/${PASS_RA}@${DB_RA}
        set heading off feedback off verify off
        SELECT '#1=' || TO_CHAR(LAST_EXTRACT_DT, 'DD/MM/YYYY') || '#'
        FROM ${id_table}
        WHERE ${id_field}=$1;
        exit;
ENDOFSQL
`
  last_extract_dt=`echo "$result" | tr -d '[[\n],[:blank:]]' | sed 's/.*#1= *//' | sed 's/ *#.*//'`
  if [ "${last_extract_dt}" ]; then
           echo "    => Last Extract Date: [${last_extract_dt}]"
           echo "    => Last Extract Date: [${last_extract_dt}]" >> $logfile
           echo ""
           echo "" >> $logfile
	   echo "  Starting the extract..."
	   echo "  Starting the extract..." >> $logfile
  	   echo "  --------------------------------" 
  	   echo "  --------------------------------" >> $logfile
  else
           echo "    => ERROR Getting the last extract date. Extract process aborted."
           echo "    => ERROR Getting the last extract date. Extract process aborted." >> $logfile
           abort=1
  fi

}

#-----------------------------------------------------------------------------------------

function update_last_extract_dt {

  echo "  Updating Last Extract Date..."
  echo "  Updating Last Extract Date..." >> $logfile

  result=`sqlplus -s /NOLOG << ENDOFSQL
         connect ${UID_RA}/${PASS_RA}@${DB_RA}
         UPDATE ${id_table}
         SET LAST_EXTRACT_DT=TRUNC(SYSDATE, 'DD')
         WHERE ${id_field}=$1;
         COMMIT;
         exit;
ENDOFSQL
`
}

#-----------------------------------------------------------------------------------------

function finish_log_and_end {

  echo "  --------------------------------"
  echo "  --------------------------------" >> $logfile
  echo "  Done."
  echo "  Done." >> $logfile

  echo ""
  echo "" >> $logfile

  if [[ $running_mode = "interactive" ]];then
     echo "Press Enter to return to the main menu"
     read unused
  fi

}

#-----------------------------------------------------------------------------------------

function calculate_next_date {

calculated_day=$(perl -e '
        $d = time();
        $d+=(86400*$ARGV[0]);
        @f = localtime($d);
        printf "%04d%02d%02d\n", $f[5]+1900, $f[4]+1, $f[3];
        ' $1)

}

#-----------------------------------------------------------------------------------------

function read_resource_file {

nbsources=0;
nbsubsources=800;
while read line
do
   if  [[ $line = +(#)* ]];then
        # Do nothing. It's a comment
        unused=1
   else
        if [[ $line = *:* ]];then
           # Parse the parameter given
           parametername=${line%%:*};
           parametername=${parametername##+([[:blank:]])};
           parametervalue=${line#*:*};
           parametervalue=${parametervalue##+([	, ])};
           case $parametername in
                "source-id" )   sourceid=$parametervalue; ((nbsources=nbsources+1));  ;;
                "sub-source-id" ) parentsourceid=${parametervalue%.*};
                                  ((nbsubsources=nbsubsources+1));
				  sourceid=$nbsubsources;
                                  subsourcekey=${parametervalue#*.};
                                  subsourceid[${nbsubsources}]=${subsourcekey};
                                  parentsource[${nbsubsources}]=${parentsourceid};;
                "source-name" ) source_name[${sourceid}]="${parametervalue}";  ;;
                "extract-method" ) extract_method[${sourceid}]="${parametervalue}" ;;
                "main-directory" ) main_directory[${sourceid}]="${parametervalue}";;
                "source-directory" ) source_directory[${sourceid}]="${parametervalue}";;
                "remote-host" ) remote_host[${sourceid}]="${parametervalue}";  ;;
                "remote-user" ) remote_user[${sourceid}]="${parametervalue}";  ;;
                "remote-password" ) remote_password[${sourceid}]="${parametervalue}";  ;;
		"remote-port" ) remote_port[${sourceid}]="${parametervalue}"; ;;
		"remote-delete" ) remote_delete[${sourceid}]="${parametervalue}"; ;;
		"no-duplicate-check" ) no_duplicate_check[${sourceid}]="${parametervalue}"; ;;
                "rsh-allowed" ) rsh_allowed[${sourceid}]="${parametervalue}";  ;;
		"remote-directory" ) remote_directory[${sourceid}]="${parametervalue}";  ;;
                "file-pattern" ) file_pattern[${sourceid}]="${parametervalue}";  ;;
                "remote-directory" ) remote_directory[${sourceid}]="${parametervalue}";  ;;
		"parsing-prg" ) parsing_prg[${sourceid}]="${parametervalue}";  ;;
		"oracle-table" ) oracle_table[${sourceid}]="${parametervalue}";  ;;
                "ORACLE-UID" ) ORACLE_UID[${sourceid}]="${parametervalue}";  ;;
                "ORACLE-PASS" ) ORACLE_PASS[${sourceid}]="${parametervalue}";  ;;
                "ORACLE-DB" ) ORACLE_DB[${sourceid}]="${parametervalue}";  ;;
                "SQLS-UID" ) SQLS_UID[${sourceid}]="${parametervalue}";  ;;
                "SQLS-PASS" ) SQLS_PASS[${sourceid}]="${parametervalue}";  ;;
                "SQLS-URL" ) SQLS_URL[${sourceid}]="${parametervalue}";  ;;
		"result-file" ) result_file[${sourceid}]="${parametervalue}";  ;;
           esac
        else
            # Do nothing. It's an invalid entry
            unused=1
        fi
   fi
done < ./resource.dat

}

#-----------------------------------------------------------------------------------------

function display_sources {

  clear

  echo ""; echo ""; echo "";
  echo "    ###############################################";
  echo "    #                                             #";
  echo "    #            - REVENUE ASSURANCE -            #";
  echo "    #${module_description}#";
  echo "    #                                             #";
  echo "    #${SOURCE}#";
  echo "    #                                             #";
  echo "    ###############################################";
  echo "";
  echo ""
  echo "Available Data Sources:";
  echo "----------------------";
  sourceid=1;
  while [[ $sourceid -le $nbsources ]]; do
     echo "  $sourceid ) ${source_name[${sourceid}]}"
     ((sourceid=sourceid+1));
  done
  echo ""
  echo "  0) Cancel"
  echo ""
  echo "Select your option: "

}

#-----------------------------------------------------------------------------------------

function display_reconciliations {

  clear

  echo ""; echo ""; echo "";
  echo "    ###############################################";
  echo "    #                                             #";
  echo "    #            - REVENUE ASSURANCE -            #";
  echo "    #${module_description}#";
  echo "    #                                             #";
  echo "    #         (Reconcile Platform Loads)          #";
  echo "    #                                             #";
  echo "    ###############################################";
  echo "";
  echo ""
  echo "Available Reconciliations:";
  echo "--------------------------";
  reconciliationid=1;
  while [[ $reconciliationid -le $nb_reconciliations ]]; do
     echo "  $reconciliationid ) ${Reconciliation[${reconciliationid}]}"
     ((reconciliationid=reconciliationid+1));
  done
  echo ""
  echo "  0) Cancel"
  echo ""
  echo "Select your option: "

}

#-----------------------------------------------------------------------------------------


function read_cron_file {

  cron_file=tasks_cron.dat

  nb_tasks=0
  while read line ; do
        ((nb_tasks=nb_tasks+1))
        task_command[${nb_tasks}]="`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\1/'`"
        task_time[${nb_tasks}]=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\2/'`
        task_frequency[${nb_tasks}]=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\3/'`
  done < ./${cron_file}


}

#-----------------------------------------------------------------------------------------

function read_reconciliations_file {
 
  reconciliation_file=reconciliations.dat

  nb_reconciliations=0
  while read line ; do
	if [[ $line = *\;*\;* ]];then
		line=${line%%#*};
		line=${line##+([[:blank:]])};
        	((nb_reconciliations=nb_reconciliations+1))
        	ReconciliationId[${nb_reconciliations}]="`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\1/'`"
        	Platform_A[${nb_reconciliations}]=${source_name[`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\2/'`]}
		sourceid=`echo $line | sed 's/\(.*\);\(.*\);\(.*\)/\3/'`
		if [[ $sourceid -gt 0 ]]; then
        		Platform_B[${nb_reconciliations}]=${source_name[$sourceid]}
			Reconciliation[${nb_reconciliations}]="Reconciliation ${Platform_A[${nb_reconciliations}]} - ${Platform_B[${nb_reconciliations}]}"
		else
        		Platform_B[${nb_reconciliations}]=""
			Reconciliation[${nb_reconciliations}]="${Platform_A[${nb_reconciliations}]} (Intra-Platform Reconciliation)"
		fi
	fi
   done < ./${reconciliation_file}

}


#-----------------------------------------------------------------------------------------

function purge_logfiles {

  for myfile in $(find ${logfile_dir} -mtime +62 ); do
     if [[ -d  $myfile ]];then
        # This is a directory.. skip
        unused=1
     else
        rm -f $myfile
     fi
  done

  for myfile in $(find ${main_dir}/bin/sqlldr/log -mtime +62 ); do
     if [[ -d  $myfile ]];then
        # This is a directory.. skip
        unused=1
     else
       rm -f $myfile
     fi
  done
}

#-----------------------------------------------------------------------------------------
