#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

. ../Common/lib.ksh

#identify if we need to run in batch mode or interactive mode
if [[ $# = 0 ]];then
  running_mode=interactive
else
  running_mode=batch
fi

MODULE="SDR"
init_global_variables
read_resource_file 
read_reconciliations_file

# Oracle Reconciling Procedures
oracle_proc_name=P_SDR_LOAD_COMPARE

#############################
function get_last_load_number {

  result=`sqlplus -s /NOLOG << ENDOFSQL
	   connect ${UID_RA}/${PASS_RA}@${DB_RA}
           set heading off feedback off verify off
		select '#1=', a.last_load_nb, '#2=', b.last_load_nb, '#'
		from ref_sdr_reconciliations x, ref_sdr_platforms a, ref_sdr_platforms b
		where reconciliation_id=${ReconciliationId[$1]}
		and platform_id_a=a.platform_id
		and platform_id_b=b.platform_id(+);
                exit;
ENDOFSQL
`

  load_A=`echo $result | sed 's/.*#1= ** *//' | sed 's/ ** *#.*//'`
  if [[ -n "${Platform_B[$1]}" ]]; then
	load_B=`echo $result | sed 's/.*#2= ** *//' | sed 's/ ** *#.*//'`
   else
        load_B=null
   fi

}
#############################

function get_load_number {

clear

echo ""; echo ""; echo ""
echo "    ###############################################"
echo "    #                                             #"
echo "    #            - REVENUE ASSURANCE -            #"
echo "    #              Platform Integrity             #"
echo "    #                                             #"
echo "    #            (Reconcile Platform Loads)       #"
echo "    #                                             #"
echo "    ###############################################"

echo ""
echo ""
echo "Enter the Platform Loads:"
echo "-------------------------"

correct_load=0
while [ $correct_load -ne 1 ]
do
   echo "${Platform_A[$1]}:"
   read load_A
   correct_load=1
done

if [[ -n "${Platform_B[$1]}" ]]; then
	echo ""
	correct_load=0
	while [ $correct_load -ne 1 ]
	do
   		echo "${Platform_B[$1]}:"
   		read load_B
   		correct_load=1
	done
else
 load_B=null
fi

clear

}

#############################

function reconcile {
 
  if [[ $running_mode = "interactive" ]];then
     get_load_number $1
  else
     get_last_load_number $1
  fi
  abort=0

  # Prepare name of the log file
  date=$(date +%Y%m%d_%H%M%S)
  logfile=${logfile_dir}/reconciling_SDR_${date}.log
  touch $logfile

  echo "##############################"  
  echo "##############################"  >> $logfile
  echo ${Reconciliation[$1]} 
  echo ${Reconciliation[$1]} >> $logfile
  echo "##############################" 
  echo "##############################" >> $logfile
  echo "  Starting the Platform Load Reconciliation..." 
  echo "  Starting the Platform Load Reconciliation..." >> $logfile
  echo "  --------------------------------------------" 
  echo "  --------------------------------------------" >> $logfile

  validate_RA_DB_available
  if [ $abort = 0 ]; then
  	# No Error - Continue
 	# Execute the SQL-Procedures to compare Platform Loads
	echo "  Inserting a new row in RESULT_SDR_RECON_SUMMARY" 
	echo "  Inserting a new row in RESULT_SDR_RECON_SUMMARY" >> $logfile
	result=`sqlplus -s /NOLOG << ENDOFSQL
	connect ${UID_RA}/${PASS_RA}@${DB_RA}
	DELETE RESULT_SDR_RECON_SUMMARY
	WHERE RECONCILIATION_ID=${ReconciliationId[$1]}
	AND LOAD_NB_A=${load_A}
	AND ((LOAD_NB_B=${load_B}) OR ((${load_B} IS NULL) AND (LOAD_NB_B IS NULL)));

	INSERT INTO RESULT_SDR_RECON_SUMMARY (
		RECONCILIATION_ID,
		LOAD_NB_A,
		LOAD_NB_B,
		STATUS,
		PROCESS_DT)
	VALUES(
   		${ReconciliationId[$1]},
		${load_A},
		${load_B},
		1,
		SYSDATE
	);
	COMMIT;
ENDOFSQL
`

	parameters="RECON_ID => ${ReconciliationId[$1]}"
	parameters="$parameters, LOAD_PLATFORM_A => ${load_A}"
	if [[ -n "${Platform_B[$1]}" ]]; then
		parameters="$parameters, LOAD_PLATFORM_B => ${load_B}"
	fi
	formated_params=`echo $parameters | sed 's/,  */,\\\n		/g'`

	echo "  Executing ${oracle_proc_name}($formated_params)"
	echo "  Executing ${oracle_proc_name}($formated_params)" >> $logfile
	result=`sqlplus -s /NOLOG << ENDOFSQL
        connect ${UID_RA}/${PASS_RA}@${DB_RA}
	execute ${oracle_proc_name}($parameters);
	exit;
ENDOFSQL
`
	#look for Oracle error codes 
	search_and_log_ORACLE_error ${DB_RA}
	if [ $abort = 1 ]; then

		result=`sqlplus -s /NOLOG << ENDOFSQL
		connect ${UID_RA}/${PASS_RA}@${DB_RA}
		UPDATE RESULT_SDR_RECON_SUMMARY
		SET STATUS=-1,
		    PROCESS_DT=SYSDATE
		WHERE RECONCILIATION_ID=${ReconciliationId[$1]} 
		AND LOAD_NB_A=${load_A}
		AND ((LOAD_NB_B=${load_B}) OR ((${load_B} IS NULL) AND (LOAD_NB_B IS NULL)));

		COMMIT;
ENDOFSQL
`

  	else
	  # No Error
		echo "  Updating SDR_RECON_SUMMARY" 
		echo "  Updating SDR_RECON_SUMMARY"  >> $logfile
		result=`sqlplus -s /NOLOG << ENDOFSQL
		connect ${UID_RA}/${PASS_RA}@${DB_RA}
		UPDATE RESULT_SDR_RECON_SUMMARY
		SET STATUS=2,
		    PROCESS_DT=SYSDATE
		WHERE RECONCILIATION_ID=${ReconciliationId[$1]} 
		AND LOAD_NB_A=${load_A}
		AND ((LOAD_NB_B=${load_B}) OR ((${load_B} IS NULL) AND (LOAD_NB_B IS NULL)));

		COMMIT;
ENDOFSQL
`
		echo "    => Success"
		echo "    => Success" >> $logfile
	fi
  fi

  echo "  -------------------------------------"
  echo "  -------------------------------------" >> $logfile
  echo "  Done."
  echo "  Done." >> $logfile

  echo ""
  echo "" >> $logfile
  if [[ $running_mode = "interactive" ]];then
     echo "Press Enter to return to the main menu"
     read unused
  fi


}

#############################

choice=-1

if [[ $running_mode = "batch" ]];then
   reconcile_option $1
else

   while [ $choice -ne 0 ]
   do

	display_reconciliations
	read choice

    	if [[ $choice = 0 ]]; then
        	clear;
        	exit 1;
    	fi
    	if [[ $choice == +([0-9]) ]]; then
        	if [[ $choice -le $nb_reconciliations ]]; then
			reconcile $choice
        	else
           		clear;
           		echo "Wrong option entered";
           		echo "Press Enter";
           		read unused;
           		choice=-1;
        	fi
    	else
        	clear;
        	echo "Wrong option entered";
        	echo "Press Enter";
        	read unused;
       		choice=-1;
    	fi
   done

fi

