#!/usr/bin/ksh

#-----------------------------------------------------------------------------------------
#--
#-- Copyright (c) 2009-2012 ProactiveRA
#--
#-- This script is free software; you can redistribute it and/or modify it under
#-- the terms of the GNU Lesser General Public License as published by the Free
#-- Software Foundation; either version 3 of the License, or (at your option)
#-- any later version.
#--
#-- This script is distributed in the hope that it will be useful, but WITHOUT
#-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#-- FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
#-- details.
#--
#-- You should have received a copy of the GNU Lesser General Public License
#-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
#--
#-----------------------------------------------------------------------------------------

. ./lib.ksh

while read line ; do
        PASS_RA=$line;
done < .PASS_RA

while read line ; do
        PASS_LIFERAY=$line;
done < .PASS_LIFERAY

param_files[1]=structure_only.dat
param_title[1]=STRUCTURE
param_files[2]=reference_tables.dat
param_title[2]=REF
param_files[3]=biv_tables.dat
param_title[3]=BIV
param_files[4]=cdr_summary_tables.dat
param_title[4]=CDR_SUMMARY
param_files[5]=cdr_tables.dat
param_title[5]=CDR
param_files[6]=slice_tables.dat
param_title[6]=SLICE
param_files[7]=structure_dbd_import.dat
param_title[7]=STRUCTURE_DBD
param_files[8]=kpi_tables.dat
param_title[8]=KPI
param_files[9]=sdr_tables.dat
param_title[9]=SDR

#############################

function import_db {

date=$(date +%Y%m%d_%H%M%S)
logfile=import_${date}.log


  # Verify availability of database before attempting loading anything
  result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_RA}/${PASS_RA}@${DB_RA}
  set heading off feedback off verify off
  select sysdate from dual;
  exit;
ENDOFSQL
`

  error_found=`echo $result | tr -d '[\n]' | ggrep -e 'ORA' -e 'SP' | sed 's/.*ORA-/    => ORA-/'  | sed 's/.*SP/    => SP/'`
  if [ "$error_found" ]
  then
  # Error found... Log the Error and skip this step
	echo "    => ERROR connecting to the database. Uploading skipped."
	echo "    => ERROR connecting to the database. Uploading skipped." >> log/$logfile 
	echo "$error_found"
	echo "$error_found" >> log/$logfile
  else
  # No Error - Continue


result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_RA}/${PASS_RA}@${DB_RA}
  BEGIN  
      FOR c IN  (SELECT c.owner, c.table_name, c.constraint_name   
	         FROM user_constraints c, user_tables t   
	         WHERE c.table_name = t.table_name   
	         AND c.status = 'ENABLED'   
 	         ORDER BY c.constraint_type DESC)  
      LOOP    
	 dbms_utility.exec_ddl_statement('alter table ' || c.owner || '.' || c.table_name || ' disable constraint ' || c.constraint_name);  
      END LOOP;
   END;
   /

  BEGIN
      FOR c IN  (SELECT c.table_name, c.index_name
                 FROM user_indexes c, user_tables t
                 WHERE c.table_name = t.table_name
                 AND c.status = 'VALID'
		 AND c.index_type != 'LOB'
                 ORDER BY c.index_type DESC)
      LOOP
         dbms_utility.exec_ddl_statement('alter index ' || c.index_name || ' disable' );
      END LOOP;
   END;
   /

  exit;
ENDOFSQL
`

while read line ; do
        TABLE=$line;

echo "    => Truncating ${TABLE}"
echo "    => Truncating ${TABLE}" >> log/$logfile

# Truncate the temporary tables
  result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_RA}/${PASS_RA}@${DB_RA}
  truncate table ${TABLE};
  exit;
ENDOFSQL
`
done < param_files/${param_title[$1]}.tables


fi

for myfile in $(ls data_files/export_${param_title[$1]}_*.dat 2> /dev/null); do

if [[ -d  $myfile ]];then
	# This is a directory.. skip
       	unused=1
     	else

imp ${UID_RA}/${PASS_RA}@${DB_RA} PARFILE=param_files/${param_files[$1]} FILE=${myfile} LOG=log/${logfile}

fi
done

result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_RA}/${PASS_RA}@${DB_RA}
  BEGIN 
      FOR c IN  (SELECT c.owner, c.table_name, c.constraint_name  
                 FROM user_constraints c, user_tables t  
                 WHERE c.table_name = t.table_name  
                 AND c.status = 'ENABLED'  
                 ORDER BY c.constraint_type DESC) 
      LOOP   
         dbms_utility.exec_ddl_statement('alter table ' || c.owner || '.' || c.table_name || ' enable constraint ' || c.constraint_name);
      END LOOP;
   END;
   /

  BEGIN
      FOR c IN  (SELECT c.table_name, c.index_name
                 FROM user_indexes c, user_tables t
                 WHERE c.table_name = t.table_name
                 AND c.status = 'VALID'
                 ORDER BY c.index_type DESC)
      LOOP
         dbms_utility.exec_ddl_statement('alter index ' || c.index_name || ' enable' );
      END LOOP;
   END;
   /

  exit;
ENDOFSQL
`


echo "  --------------------------------"
echo "  Done."
echo "Press Enter";

read unused
}

#############################

function import_dashboard_db {

date=$(date +%Y%m%d_%H%M%S)
logfile=import_${date}.log


  # Verify availability of database before attempting loading anything
  result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_LIFERAY}/${PASS_LIFERAY}@${DB_RA}
  set heading off feedback off verify off
  select sysdate from dual;
  exit;
ENDOFSQL
`

  error_found=`echo $result | tr -d '[\n]' | ggrep -e 'ORA' -e 'SP' | sed 's/.*ORA-/    => ORA-/'  | sed 's/.*SP/    => SP/'`
  if [ "$error_found" ]
  then
  # Error found... Log the Error and skip this step
        echo "    => ERROR connecting to the database. Uploading skipped."
        echo "    => ERROR connecting to the database. Uploading skipped." >> log/$logfile 
        echo "$error_found"
        echo "$error_found" >> log/$logfile
  else
  # No Error - Continue

result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_LIFERAY}/${PASS_LIFERAY}@${DB_RA}
  BEGIN  
      FOR c IN  (SELECT c.owner, c.table_name, c.constraint_name   
                 FROM user_constraints c, user_tables t   
                 WHERE c.table_name = t.table_name   
                 AND c.status = 'ENABLED'   
                 ORDER BY c.constraint_type DESC)  
      LOOP    
         dbms_utility.exec_ddl_statement('alter table ' || c.owner || '.' || c.table_name || ' disable constraint ' || c.constraint_name);  
      END LOOP;
   END;
   /

  exit;
ENDOFSQL
`
echo $result

while read line ; do
        TABLE=$line;

echo "    => Dropping Table ${TABLE}"
echo "    => Dropping Table ${TABLE}" >> log/$logfile

# Drop the tables
  result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_LIFERAY}/${PASS_LIFERAY}@${DB_RA}
  drop table ${TABLE} cascade constraints;
  exit;
ENDOFSQL
`
echo $result
done < param_files/${param_title[$1]}.tables


fi

for myfile in $(ls data_files/export_${param_title[$1]}_*.dat 2> /dev/null); do

if [[ -d  $myfile ]];then
        # This is a directory.. skip
        unused=1
        else

imp ${UID_LIFERAY}/${PASS_LIFERAY}@${DB_RA} PARFILE=param_files/${param_files[$1]} FILE=${myfile} LOG=log/${logfile}

fi
done

result=`sqlplus -s /NOLOG << ENDOFSQL
  connect ${UID_LIFERAY}/${PASS_LIFERAY}@${DB_RA}
  BEGIN  
      FOR c IN  (SELECT c.owner, c.table_name, c.constraint_name
                 FROM user_constraints c, user_tables t
                 WHERE c.table_name = t.table_name
                 AND c.status = 'DISABLED'   
                 ORDER BY c.constraint_type DESC)
      LOOP     
         dbms_utility.exec_ddl_statement('alter table ' || c.owner || '.' || c.table_name || ' enable constraint ' || c.constraint_name);
      END LOOP;
   END;
   /

  exit;
ENDOFSQL
`

echo "  --------------------------------"
echo "  Done."
echo "Press Enter";

read unused
}

#############################



choice=-1


while [ $choice -ne 0 ]
do

clear
echo ""; echo ""; echo ""
echo "    ###############################################"
echo "    #                                             #"
echo "    #            - REVENUE ASSURANCE -            #"
echo "    #                Slice Utility                #"
echo "    #                                             #"
echo "    ###############################################"
echo ""
echo ""
echo "Available Options:"
echo "------------------"
echo "  1)  Database structure of RA user (All Objects, No Rows)"
echo "  2)  Configuration tables (REF_)"
echo "  3)  Billing Validation Tables (BIV_)"
echo "  4)  CDR Summary Stats (CDR_ALL*)"
echo "  5)  CDR tables (CDR_)"
echo "  6)  Custom List of Tables (SLICE)"
echo "  7)  Database structure of LIFERAYPRD user (All Objects, All Data)"
echo "  8)  KPI tables (*KPI*)"
echo "  9)  SDR tables (*KPI*)"

echo ""
echo "  0) Cancel"
echo ""
echo "Select your option: "
read choice


if [[ $choice = 0 ]]; then
        clear;
        exit 1;
fi
if [[ $choice == +([0-9]) ]]; then
   if [[ $choice -lt 10 ]]; then
        if [[ $choice = 7 ]]; then
                import_dashboard_db $choice;
        else
        	import_db $choice;
	fi
   else
        clear;
        echo "Wrong option entered";
        echo "Press Enter";
        read unused;
        choice=-1;
   fi
else
   clear;
   echo "Wrong option entered";
   echo "Press Enter";
   read unused;
   choice=-1;
fi

done
